//
//  Bridge.js
//  FolioReader-Android
//
//  Created by Heberti Almeida on 06/05/15.
//  Copyright (c) 2015 Folio Reader. All rights reserved.
//

var gCurrHighlightElement;
var gDirection;

var Direction = Object.freeze({
    VERTICAL: "VERTICAL",
    HORIZONTAL: "HORIZONTAL"
});

// Class manipulation
function hasClass(ele, cls) {
    return !!ele.className.match(new RegExp('(\\s|^)' + cls + '(\\s|$)'));
}

function addClass(ele, cls) {
    if (!hasClass(ele, cls)) ele.className += " " + cls;
}

function removeClass(ele, cls) {
    if (hasClass(ele, cls)) {
        var reg = new RegExp('(\\s|^)' + cls + '(\\s|$)');
        ele.className = ele.className.replace(reg, ' ');
    }
}

$(function () {
    window.ssReader = Class({
        $singleton: true,

        init: function () {
            rangy.init();

            this.highlighter = rangy.createHighlighter();

            this.highlighter.addClassApplier(rangy.createClassApplier("underline", {
                ignoreWhiteSpace: true,
                tagNames: ["span", "a"]
            }));

            this.highlighter.addClassApplier(rangy.createClassApplier("note", {
                ignoreWhiteSpace: true,
                tagNames: ["span", "a"]
            }));
        },

        base64encode: function (str) {
            return btoa(unescape(encodeURIComponent(str)));
        },

        base64decode: function (str) {
            return decodeURIComponent(escape(atob(str)));
        },

        clearSelection: function () {
            if (window.getSelection) {
                if (window.getSelection().empty) {  // Chrome
                    window.getSelection().empty();
                } else if (window.getSelection().removeAllRanges) {  // Firefox
                    window.getSelection().removeAllRanges();
                }
            } else if (document.selection) {  // IE?
                document.selection.empty();
            }
        },

        highlightSelection: function (color) {
            try {

                this.highlighter.highlightSelection(color, null);
                var range = window.getSelection().toString();
                var params = {content: range, rangy: this.getHighlights(), color: color};
                this.clearSelection();
                return JSON.stringify(params);
            } catch (err) {
                console.log("highlightSelection : " + err);
            }
        },

        unHighlightSelection: function () {
            try {
                this.highlighter.unhighlightSelection();
                return true;
            } catch (err) {
            }
        },

        removeHighlight: function(highlight) {
            var highlights = [highlight];
            this.highlighter.removeHighlights(highlights);
        },

        getHighlights: function () {
            try {
                return this.highlighter.serialize();
            } catch (err) {
            }
        },

        setHighlights: function (serializedHighlight) {
            try {
                this.highlighter.removeAllHighlights();
                this.highlighter.deserialize(serializedHighlight);
            } catch (err) {
            }
        },

        removeAll: function () {
            try {
                this.highlighter.removeAllHighlights();
            } catch (err) {
            }
        },
    });

    if (typeof ssReader !== "undefined") {
        ssReader.init();
    }
});

/**
 * Returns JSON of selection rect
 * @param {Element} [element]
 * @returns {object} JSON of {@link DOMRect}
 */
function getSelectionRect(element) {
    console.log("-> getSelectionRect");

    var range;
    if (element !== undefined) {
        range = document.createRange();
        range.selectNodeContents(element);
    } else {
        range = window.getSelection().getRangeAt(0);
    }

    //var rect = range.getBoundingClientRect();
    var rect = RangeFix.getBoundingClientRect(range);
    return {
        left: rect.left,
        top: rect.top,
        right: rect.right,
        bottom: rect.bottom
    };
}

function clearSelection() {
    console.log("-> clearSelection");
    window.getSelection().removeAllRanges();
}

/**** fenbi ****/
$(function() {
    mockJsBridge();

    $("html").click();

    var $img = $("img");
    var imgNum = $img.length;
    if (imgNum > 0) {
        $img.each(function() {
            if (!this.complete) {
                $(this).on("load", function() {
                    imgNum--;
                    if (imgNum <= 0) onDomAndImageLoad();
                });
                return;
            }

            imgNum--;
            if (imgNum <= 0) onDomAndImageLoad();
        });
    } else {
        onDomAndImageLoad();
    }
})

function onDomAndImageLoad() {
    console.log("Test onDomAndImageLoad");
    gDirection = $("html").hasClass("directionHorizontal") ? Direction.HORIZONTAL : Direction.VERTICAL;
    gDirection = Direction.HORIZONTAL;

    if (Direction.VERTICAL == gDirection) {
        layoutVertical();
    } else {
        layoutHorizontal();
    }

    console.log("Test onDomAndImageLoad layout end:" + window.innerHeight);
}

function layoutVertical() {
    var clear = {
        "width": "",
        "padding": "0px !important"
    };
    $("html").css(clear);
    $("body").css(clear);

    $("body").wrapInner("<div id='epub'></div>");

    var padding = 20;
    var pageWidth = window.innerWidth - (2 * padding);
    $("#epub").css({
        "padding-left": padding,
        "padding-right": padding,
    });

    var epubEl = document.getElementById("epub");
    var bodyWidth = window.innerWidth;
    $("html").width(bodyWidth);
    $("body").width(bodyWidth);

    var pageCount = Math.ceil($("body").height() / window.innerHeight);

    var lineHeight = parseFloat(window.getComputedStyle(document.getElementById("epub"), null).lineHeight);
    window.swipper = new Swipper({
        "container": document.getElementById("epub"),
        "pageSize": window.innerHeight,
        "lineHeight": lineHeight,
        "direction": Direction.VERTICAL,
        "pageChange": function(page) {
            gWebView.onPageChanged(page);
        },
        "scrollChange": function(isScrollLeft, isScrollRight, isScrollTop, isScrollBottom) {
            gWebView.onScrollChanged(isScrollLeft, isScrollRight, isScrollTop, isScrollBottom);
        }
    });

    gWebView.onLayoutEnd(pageCount);
}

function layoutHorizontal() {
    var clear = {
        "width": "",
        "height": "",
        "padding": "0px !important"
    };
    $("html").css(clear);
    $("body").css(clear);

    $("body").wrapInner("<div id='epub'><div id='epubInner'></></div>");

    var padding = 20;
    var pageWidth = window.innerWidth - (2 * padding);
    $("#epubInner").css({
        "column-fill": "balance",
        "column-width": pageWidth,
        "column-gap": 2 * padding,
        "height": window.innerHeight - 2 * padding,
        "padding": padding,
    });
    // epubInner如果设置为这个属性，VivoY79等机型15页之后会显示空白, 再包一层会避免这个问题
    // 参考：https://www.tapd.cn/59745089/bugtrace/bugs/view/1159745089001044898
    $("#epub").css({
        "width": window.innerWidth,
        "overflow": "scroll",
    });

    window.swipper = new Swipper({
        "container": document.getElementById("epub"),
        "pageSize": window.innerWidth,
        "direction": Direction.HORIZONTAL,
        "pageChange": function(page) {
            gWebView.onPageChanged(page);
        },
        "scrollChange": function(isScrollLeft, isScrollRight, isScrollTop, isScrollBottom) {
            gWebView.onScrollChanged(isScrollLeft, isScrollRight, isScrollTop, isScrollBottom);
        }
    });

    gWebView.onLayoutEnd(window.swipper.pageCount);
}

function getPageIndexById(id) {
    return getPageIndexByElement(document.getElementById(id));
}

function getPageIndexByElement(element) {
    if (gDirection == Direction.HORIZONTAL) {
        console.log("getPageIndexByElement offsetLeft:" + element.offsetLeft);
        return swipper.getPageIndex(element.offsetLeft);
    } else {
        console.log("getPageIndexByElement offsetTop:" + element.offsetTop);
        return swipper.getPageIndex(element.offsetTop);
    }
}

function getPageIndex(x) {
    return Math.floor(x / window.innerWidth);
}

function getCurrPageIndex() {
    return swipper.getCurrPage();
}

function goToAnchor(anchorId) {
    console.log("goToAnchor:" + anchorId);
    var element = document.getElementById(anchorId);
    if (element) scrollToElement(element);
}

function scrollToElement(element) {
    swipper.gotoPage(getPageIndexByElement(element), true);
}

function mockJsBridge() {
    if (window.gWebView == undefined) {
        window.gWebView = {
            "onHtmlClick": function() {},
            "onLayoutEnd": function() {},
            "onPageChanged": function() {},
            "onScrollChanged": function() {},
            "onHighlightClick": function() {},
            "showLoading": function() {},
            "hideLoading": function() {}
        };
    }
}

function getSelectionText() {
    return window.getSelection().toString();
}

function getSelectionInfo() {
    var rect = getSelectionRect();

    var selection = rangy.getSelection(document);
    var ranges = selection.getAllRanges();
    var bookmark = ranges[0].getBookmark(document.body);

    var ret = {
        "rect": rect,
        "range": {
            "start": bookmark.start,
            "end": bookmark.end,
            "content": getSelectionText(),
        }
    };
    return ret;
}

function getSelectionWord() {
   var range = window.getSelection().getRangeAt(0);
   var note = window.getSelection().extentNode;
   var text = note.data;
   var reg = /^[A-Za-z]+$/;

   var word;
   var leftOffset = range.startOffset;
   var rightOffset = range.startOffset + 1;

   if(reg.test(text.charAt(range.startOffset))){
       word = text.charAt(range.startOffset);

       while(leftOffset > 0 && reg.test(text.charAt(leftOffset - 1))){
          word = text.charAt(--leftOffset) + word;
       }

       while(rightOffset < text.length && reg.test(text.charAt(rightOffset))){
          word = word + text.charAt(rightOffset ++);
       }

       // 修改文本选择区域
       range.setStart(note, leftOffset);
       range.setEnd(note, rightOffset);
   }

   var ret = {
        "word": word,
   };
   return ret;
}

function onClickHighlight(element) {
    event.stopPropagation();

    gCurrHighlightElement = element;
    var rect = getSelectionRect(element);

    gWebView.onHighlightClick(element.id, element.innerHTML, JSON.stringify(rect));
}

function removeCurrHighlight() {
    if (gCurrHighlightElement !== undefined) {
        var highlight = ssReader.highlighter.getHighlightForElement(gCurrHighlightElement);
        if (!highlight) ssReader.removeHighlight(highlight);
    }
}

function removeHighlight(elementId) {
    var element = document.getElementById(elementId);
    if (!element) return;
    var highlight = ssReader.highlighter.getHighlightForElement(element);
    if (highlight) ssReader.removeHighlight(highlight);
}

function onClickHtml() {
    console.log("onClickHtml");
    gWebView.onHtmlClick();
}

