precision mediump float;
uniform vec3 texinfo;
uniform sampler2D textureY;
uniform sampler2D textureCb;
uniform sampler2D textureCr;
varying vec2 texCoord;

mat4 rec601 = mat4(
    1.16438,  0.00000,  1.59603, -0.87079,
    1.16438, -0.39176, -0.81297,  0.52959,
    1.16438,  2.01723,  0.00000, -1.08139,
    0, 0, 0, 1
);

mat4 rec701 = mat4(
   1.164384, -0.000000,  1.792741,  -0.972945,
   1.164384, -0.213249, -0.532909,  0.301483,
   1.164384,  2.112402, -0.000000,  -1.133402,
   0.000000,  0.000000,  0.000000,  1.00000
);

float GetChromaDistUV(vec2 cbcr) {
    vec2 green = vec2(0.163389, 0.103019);
    return distance(cbcr,green);
}

vec2 SampleTextureUV(vec2 position) {
    //float y = texture2D(textureY, position).r;
    float cb = texture2D(textureCb, position).r;
    float cr = texture2D(textureCr, position).r;

    return vec2(cb, cr);
}

float GetBoxFilteredChromaDist(float cb,float cr,vec2 texCoord)
{
    vec2 pixel_size = vec2(1.0/texinfo.x,1.0/texinfo.y);    //TODO
    vec2 h_pixel_size = pixel_size / 2.0;
    vec2 point_0 = vec2(pixel_size.x, h_pixel_size.y);
    vec2 point_1 = vec2(h_pixel_size.x, -pixel_size.y);

    float distVal = GetChromaDistUV(SampleTextureUV(texCoord-point_0));
    distVal += GetChromaDistUV(SampleTextureUV(texCoord+point_0));
    distVal += GetChromaDistUV(SampleTextureUV(texCoord-point_1));
    distVal += GetChromaDistUV(SampleTextureUV(texCoord+point_1));
    distVal *= 2.0;
    distVal += GetChromaDistUV(vec2(cb,cr));

    return distVal / 9.0;
}

float saturate(float base) {
    return clamp(base,0.0,1.0);
}

/*vec4 CalcColor(vec4 rgba)
{
    vec3 gamma = vec3(1.0,1.0,1.0);
    float contrast = 1.0;
    float brightness = 0.0;

    return vec4(pow(rgba.xyz, gamma) * contrast + brightness, rgba.a);
}*/

//args
float similarity = 0.4;
float spill = 0.1;
float smoothness = 0.08;

void main() {
    float y = texture2D(textureY, texCoord).r;
    float cb = texture2D(textureCb, texCoord).r;
    float cr = texture2D(textureCr, texCoord).r;

    vec4 rgba = vec4(y, cb, cr, 1.0) * rec701;

    if(texinfo.z > 0.0) {
        float chromaDist = GetBoxFilteredChromaDist(cb,cr,texCoord);
        float baseMask = chromaDist - similarity;
        float fullMask = pow(saturate(baseMask / smoothness), 1.5);
        float spillVal = pow(saturate(baseMask / spill), 1.5);

        rgba.w *= fullMask;

        float desat = y; //saturate((rgba.x * 0.2126 + rgba.y * 0.7152 + rgba.z * 0.0722));
        rgba.xyz = vec3(desat, desat, desat) * (1.0 - spillVal) + rgba.xyz * spillVal;  //alpha

        if(rgba.w < 0.2){
            rgba.w = 0.0;
        }
    }
    gl_FragColor = rgba;
}